#ifndef L_STRING_H
#define L_STRING_H

#include <stdlib.h>

int strcasecmp(const char *s1, const char *s2);

#if 0
char *strdup(const char *s);
#endif

/* Define a L_STRING_T compatible string variable and initialize
 * name : name for the variable
 * len  : max string length for the variable
 */
#define L_STRING_DEFINE(name, len)  \
struct                              \
{                                   \
    int     max_len;                \
    char    str[len + 1];           \
} name = {len, {0}}

#define L_STRING_CAT(dest_p, src_p) L_STRING_Catx((L_STRING_T*)dest_p, src_p)

/* Allocate memory for a L_STRING_T pointer and initialize
 * ptr : the L_STRING_T pointer
 * len : the max string length for L_STRING_T
 * uid : got from macro L_MM_USER_ID or L_MM_USER_ID2
 * return : ptr
 * note : shall check whether ptr is NULL for further usage
 */
#define L_STRING_ALLOC(ptr, len, uid)                       \
({                                                  \
    ptr = (L_STRING_T*)L_MM_Malloc(sizeof(int)+len+1, uid); \
    if (ptr != NULL)                                \
    {                                               \
        ptr->max_len = len;                         \
        ptr->str[0] = '\0';                         \
    }                                               \
    ptr;                                            \
})

/* Free memory for a L_STRING_T pointer */
#define L_STRING_FREE(ptr)  L_MM_Free(ptr)

typedef struct
{
    int     max_len;    /* max length for the string */
    char    str[1];     /* buffer for the string */
} L_STRING_T;

/* FUNCTION NAME - L_STRING_Catx
 * PURPOSE : Concatenate based string with source string to form a new string.
 * INPUT   : dest_p - a L_STRING_T variable containing based string
 *           src_p  - buffer containing source string
 * OUTPUT  : dest_p->str - buffer containing new string
 * RETURN  : TRUE  - succeeded
 *           FALSE - failed
 * NOTE    : 1. if free space in dest_p->str is not enough for source string,
 *              the result is failed
 *           2. if failed, new string in dest_p->str is the same as based string
 *           3. if succeeded, new string is always null terminated
 */
BOOL_T L_STRING_Catx(L_STRING_T *dest_p, char *src_p);

#endif /* #ifndef L_STRING_H */
